@@ -1,6 +1,7 @@
 /* Swinder - Portable library for spreadsheet
    Copyright (C) 2003-2005 Ariya Hidayat <ariya@kde.org>
    Copyright (C) 2006,2009 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
+   Copyright (C) 2009,2010 Sebastian Sauer <sebsauer@kdab.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -23,9 +24,24 @@
 #include "value.h"
 #include <map>
 
+#include <QtCore/QString>
+
 namespace Swinder
 {
 
+// The minimal number of rows and columns. This is used to fill remaming rows and columns with the
+// default style what is needed cause Excel does always define the default for all rows and columns
+// while ODF does only for those that are explicit defined.
+static const uint minimumColumnCount = 1024;
+static const uint minimumRowCount = 32768;
+
+// The maximal number of rows and columns. This allows us to cut rows and columns away that would
+// not be handled by the consumer application anyway cause they reached the applications limited.
+static const uint maximalColumnCount = 32768;
+static const uint maximalRowCount = 32768; //65536
+
+class Workbook;
+
 Value errorAsValue(int errorCode);
 
 static inline unsigned long readU16(const void* p)
@@ -135,6 +151,13 @@ UString readUnicodeString(const void* data, unsigned length, unsigned maxSize =
 
 std::ostream& operator<<(std::ostream& s, Swinder::UString ustring);
 
+inline QString string(const Swinder::UString& str)
+{
+    // don't use QString::fromRawData cause it does not create a deep copy what may result in
+    // garbage if the underlying str got deleted before we are using our QString.
+    return QString(reinterpret_cast<const QChar*>(str.data()), str.length());
+}
+
 /**
   Class Record represents a base class for all other type record,
   hence do not use this class in real life.
@@ -157,7 +180,7 @@ public:
     /**
       Creates a new generic record.
     */
-    Record();
+    Record(Workbook*);
 
     /**
       Destroys the record.
@@ -167,7 +190,7 @@ public:
     /**
      * Record factory, create a new record of specified type.
      */
-    static Record* create(unsigned type);
+    static Record* create(unsigned type, Workbook *book);
 
     void setVersion(unsigned v) {
         ver = v;
@@ -208,28 +231,33 @@ public:
     bool isValid() const;
 protected:
     void setIsValid(bool isValid);
-
+    // the workbook
+    Workbook *m_workbook;
     // position of this record in the OLE stream
     unsigned stream_position;
-
     // in which version does this record denote ?
     unsigned ver;
-
+    // is the record valid?
     bool valid;
 };
 
+typedef Record*(*RecordFactory)(Workbook*);
+typedef Record*(*RecordFactoryWithArgs)(Workbook*, void*);
 
-typedef Record*(*RecordFactory)();
 class RecordRegistry
 {
 public:
     static void registerRecordClass(unsigned id, RecordFactory factory);
-    static Record* createRecord(unsigned id);
+    static void registerRecordClass(unsigned id, RecordFactoryWithArgs factory, void* args);
+    static void unregisterRecordClass(unsigned id);
+    static Record* createRecord(unsigned id, Workbook *book);
 private:
     RecordRegistry() {};
     static RecordRegistry* instance();
 
     std::map<unsigned, RecordFactory> records;
+    std::map<unsigned, RecordFactoryWithArgs> recordsWithArgs;
+    std::map<unsigned, void*> recordArgs;
 };
 
 } // namespace Swinder
