@@ -3,7 +3,7 @@
  * Copyright (C) 2002 Laurent Montel <lmontel@mandrakesoft.com>
  * Copyright (c) 2003 Lukas Tinkl <lukas@kde.org>
  * Copyright (C) 2003 David Faure <faure@kde.org>
- * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+ * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
  *
  * Contact: Suresh Chande suresh.chande@nokia.com
  *
@@ -34,6 +34,7 @@
 #include <KoXmlReader.h>
 #include <KDebug>
 
+class QLocale;
 class QDomElement;
 class QDomDocument;
 class KZip;
@@ -42,51 +43,6 @@ class KoCharacterStyle;
 class KoStyleStack;
 class KoXmlWriter;
 
-// conversion from twips (http://en.wikipedia.org/wiki/Twip)
-#define TWIP_TO_DM(tw) ((tw)*0.000176389)
-#define TWIP_TO_CM(tw) ((tw)*0.001763889)
-#define TWIP_TO_MM(tw) ((tw)*0.017638889)
-#define TWIP_TO_POINT(tw) ((tw)*0.05)
-#define TWIP_TO_INCH(tw) ((tw)*0.000694444)
-#define TWIP_TO_PI(tw) ((tw)*0.004166667)
-#define TWIP_TO_CC(tw) ((tw)*0.00389404975957)
-#define TWIP_TO_PX(tw) ((tw)*0.066798611)
-
-// conversion to twips
-#define DM_TO_TWIP(dm) ((dm)*5669.28776738)
-#define CM_TO_TWIP(cm) ((cm)*566.929098146)
-#define MM_TO_TWIP(mm) ((mm)*56.6929130287)
-#define POINT_TO_TWIP(pt) ((pt)*20.0)
-#define INCH_TO_TWIP(in) ((in)*1440.0)
-#define PI_TO_TWIP(pi) ((pi)*240.0)
-#define CC_TO_TWIP(cc) ((cc)*256.80206)
-#define PX_TO_TWIP(px) ((px)*14.970371)
-
-// EMU conversion (ECMA-376, 20.1.2.1: EMU Unit of Measurement)
-//! Converts emu value (integer or double) to cm
-#define EMU_TO_CM(emu) ((emu)/360000.0)
-
-//! Performs EMU conversion and returns string.
-inline QString EMU_TO_CM_STRING(int emu)
-{
-    QString res;
-    return res.sprintf("%3.3fcm", EMU_TO_CM(double(emu)));
-}
-
-//! Converts emu value (integer or double) to inches
-#define EMU_TO_INCH(emu) ((emu)/914400.0)
-
-//! Performs EMU conversion and returns string.
-inline QString EMU_TO_INCH_STRING(int emu)
-{
-    QString res;
-    return res.sprintf("%3.3fin", EMU_TO_INCH(double(emu)));
-}
-
-// px conversion
-#define PT_TO_PX(pt) ((pt)*1.33597222222)
-#define PX_TO_CM(px) ((px)*0.0264)
-
 //! Returns from the current block if the result of @a call is not equal to KoFilter::OK
 #define RETURN_IF_ERROR( call ) \
     { \
@@ -157,8 +113,8 @@ public:
     }
     ~AutoPtrSetter() {
         if (m_pptr && *m_pptr) {
-            *m_pptr = 0;
             delete *m_pptr;
+            *m_pptr = 0;
         }
     }
     //! Bypasses the smart pointer, and returns it, so on destruction
@@ -174,6 +130,10 @@ private:
     T** m_pptr;
 };
 
+//! Decodes boolean attribute @a value. If unspecified returns @a defaultValue.
+//! @return true unless @a value is equal to "false", "off" or "0".
+MSOOXML_EXPORT bool convertBooleanAttr(const QString& value, bool defaultValue = false);
+
 //! Loads content types from "[Content_Types].xml"
 /*! Based on information from ECMA-376, Part 1: "11.2 Package Structure".
  @return status: KoFilter::OK on success or KoFilter::WrongFormat when any unexpected and critical incompatibility occurs.
@@ -230,10 +190,16 @@ MSOOXML_EXPORT KoFilter::ConversionStatus loadThumbnail(QImage& thumbnail, KZip*
 MSOOXML_EXPORT bool ST_Lang_to_languageAndCountry(const QString& value, QString& language, QString& country);
 
 //! @return QColor value for ST_HexColorRGB (Hexadecimal Color Value) (SharedML, 22.9.2.5)
+//!         or invalid QColor if @a color is not in the expected format.
 //! @par val color value in RRGGBB hexadecimal format
 inline QColor ST_HexColorRGB_to_QColor(const QString& color)
 {
-    return QColor(QRgb(0xff000000 | color.toInt(0, 16)));
+    if (color.length() != 6)
+        return QColor();
+    bool ok;
+    const uint rgb = color.toUInt(&ok, 16);
+    return ok ? QColor(QRgb(rgb)) : QColor(); // alpha ignored
+//    return QColor(QRgb(0xff000000 | color.toInt(0, 16)));
 }
 
 //! @return QBrush value for ST_HighlightColor
@@ -242,6 +208,12 @@ inline QColor ST_HexColorRGB_to_QColor(const QString& color)
 //! @par colorName named text highlight color like "black", "blue" (17.18.40)
 MSOOXML_EXPORT QBrush ST_HighlightColor_to_QColor(const QString& colorName);
 
+//! @return QColor value for DefaultIndexColor
+MSOOXML_EXPORT QColor defaultIndexedColor( int index );
+
+//! @return QLocale for the give language id
+MSOOXML_EXPORT QLocale localeForLangId( int langid );
+
 //! Converts value for 22.9.2.9 ST_Percentage (Percentage Value with Sign) from string
 //! Sets @arg ok to true on success.
 MSOOXML_EXPORT double ST_Percentage_to_double(const QString& val, bool& ok);
@@ -255,10 +227,11 @@ struct MSOOXML_EXPORT DoubleModifier {
     DoubleModifier(double v) : value(v), valid(true) {}
     DoubleModifier() : value(0.0), valid(false) {}
     double value;
-bool valid : 1;
+    bool valid;
 };
 
-MSOOXML_EXPORT QColor colorForLuminance(const QColor& color, const DoubleModifier& modulation, const DoubleModifier& offset);
+MSOOXML_EXPORT QColor colorForLuminance(const QColor& color,
+    const DoubleModifier& modulation, const DoubleModifier& offset);
 
 //! Converts shape types from ECMA-376 to ODF.
 /*! @return "Common Presentation Shape Attribute" value (ODF 1.1., 9.6.1)
@@ -272,13 +245,6 @@ MSOOXML_EXPORT QColor colorForLuminance(const QColor& color, const DoubleModifie
 //! CASE #P500
 MSOOXML_EXPORT QByteArray ST_PlaceholderType_to_ODF(const QByteArray& ecmaType);
 
-//! Converts EMU Unit of Measurement to cm.
-/*! Converts value expressed in EMU (ECMA-376, 20.1.2.1: EMU Unit of Measurement)
-    to ODF-compliant "0.000cm" unit.
-    @return empty string on error. */
-//! CASE #P505
-MSOOXML_EXPORT QString EMU_to_ODF_CM(const QString& emuValue);
-
 //! Sets up @a textStyleProperties with underline style matching MSOOXML name @a msooxmlName.
 //! Based on 17.18.99 ST_Underline (Underline Patterns), WML ECMA-376 p.1681
 //! and on 20.1.10.82 ST_TextUnderlineType (Text Underline Types), DrawingML ECMA-376 p.3450 (merged)
@@ -287,8 +253,8 @@ MSOOXML_EXPORT void setupUnderLineStyle(const QString& msooxmlName, KoCharacterS
 //! A helper allowing to buffer xml streams and writing them back later
 /*! This class is useful when information that has to be written in advance is based
     on XML elements parsed later. In such case the information cannot be saved in one pass.
-    Example of this is paragtaphs style name: is should be written to style:name attribute but
-    relevant XML elements (that we use for buildingthe style) are appearing later.
+    Example of this is paragraphs style name: is should be written to style:name attribute but
+    relevant XML elements (that we use for building the style) are appearing later.
     So we first output created XML to a buffer, then save the parent element with the style name
     and use KoXmlWriter::addCompleteElement() to redirect the buffer contents as a subelement.
 
@@ -298,8 +264,16 @@ MSOOXML_EXPORT void setupUnderLineStyle(const QString& msooxmlName, KoCharacterS
      XmlWriteBuffer buf;
      body = buf.setWriter(body);
      // ...
-     // ... buf.originalWriter() can be used here
+     // buf.originalWriter() can be used here ...
      // ...
+     // Use the new buffered body writer here, e.g.:
+     body->startElement("text:span", false);
+     body->addAttribute("text:style-name", currentTextStyleName);
+     body->addTextSpan(text);
+     body->endElement();
+     // We are done with the buffered body writer, now release it and restore the original body writer.
+     // This inserts all the XML buffered by buf into the original body writer
+     // (internally using KoXmlWriter::addCompleteElement()).
      body = buf.releaseWriter();
      @endcode */
 class MSOOXML_EXPORT XmlWriteBuffer
@@ -319,9 +293,11 @@ public:
     KoXmlWriter* setWriter(KoXmlWriter* writer);
 
     //! Releases the original writer set before using setWriter(KoXmlWriter*&).
-    /*! @return the original writer set in setWriter();
-                this writer usually should be assigned back to the variable
-                altered by the recent use of setWriter(). */
+    /*! This inserts all the XML buffered by buffer into the original body writer passed in setWriter()
+     (internally using KoXmlWriter::addCompleteElement()).
+     @return the original writer set in setWriter();
+             this writer usually should be assigned back to the variable
+             altered by the recent use of setWriter(). */
     KoXmlWriter* releaseWriter();
 
     //! @return the original writer set in setWriter(). Does not change the state of the buffer.
@@ -329,6 +305,9 @@ public:
     KoXmlWriter* originalWriter() const {
         return m_origWriter;
     }
+
+    //! Clears this buffer without performing any output to the writer.
+    void clear();
 private:
     //! Internal, used in releaseWriter() and the destructor; Does not assert when there's nothing to release.
     KoXmlWriter* releaseWriterInternal();
