@@ -81,10 +81,20 @@
 #include <KoOdfStylesReader.h>
 #include <KoXmlWriter.h>
 
+#include <KoTextLoader.h>
+#include <KoStyleManager.h>
+#include <KoTextSharedLoadingData.h>
+#include <KoTextDocument.h>
+#include <KoTextWriter.h>
+#include <KoEmbeddedDocumentSaver.h>
+#include <KoParagraphStyle.h>
+
 #include <kdebug.h>
 #include <KNotification>
 
 #include <QTimer>
+#include <QTextDocument>
+#include <QTextCursor>
 
 using namespace KSpread;
 
@@ -259,7 +269,7 @@ QString Cell::columnName() const
 // static
 QString Cell::columnName(uint column)
 {
-    if (column < 1 || column > KS_colMax)
+    if (column < 1)     //|| column > KS_colMax)
         return QString("@@@");
 
     QString   str;
@@ -321,7 +331,7 @@ Style Cell::effectiveStyle() const
 {
     Style style = sheet()->cellStorage()->style(d->column, d->row);
     // use conditional formatting attributes
-    if (Style* conditialStyle = conditions().testConditions(*this))
+    if (Style* conditialStyle = conditions().testConditions(*this, sheet()->map()->styleManager()))
         style.merge(*conditialStyle);
     return style;
 }
@@ -372,6 +382,9 @@ void Cell::setUserInput(const QString& string)
         // set the value
         sheet()->cellStorage()->setUserInput(d->column, d->row, string);
     }
+
+    // remove any existing richtext
+    setRichText(QSharedPointer<QTextDocument>());
 }
 
 
@@ -414,6 +427,17 @@ void Cell::setValue(const Value& value)
     sheet()->cellStorage()->setValue(d->column, d->row, value);
 }
 
+
+QSharedPointer<QTextDocument> Cell::richText() const
+{
+    return sheet()->cellStorage()->richText(d->column, d->row);
+}
+
+void Cell::setRichText(QSharedPointer<QTextDocument> text)
+{
+    sheet()->cellStorage()->setRichText(d->column, d->row, text);
+}
+
 // FIXME: Continue commenting and cleaning here (ingwa)
 
 
@@ -586,7 +610,7 @@ QString Cell::encodeFormula(bool fixedReferences) const
 
 QString Cell::decodeFormula(const QString &_text) const
 {
-    QString erg = "";
+    QString erg;
     unsigned int pos = 0;
     const unsigned int length = _text.length();
 
@@ -1031,7 +1055,7 @@ QString Cell::saveOdfCellStyle(KoGenStyle &currentCellStyle, KoGenStyles &mainSt
     const Conditions conditions = this->conditions();
     if (!conditions.isEmpty()) {
         // this has to be an automatic style
-        currentCellStyle = KoGenStyle(KoGenStyle::StyleAutoTableCell, "table-cell");
+        currentCellStyle = KoGenStyle(KoGenStyle::TableCellAutoStyle, "table-cell");
         conditions.saveOdfConditions(currentCellStyle);
     }
     return style().saveOdf(currentCellStyle, mainStyles, d->sheet->map()->styleManager());
@@ -1122,7 +1146,7 @@ bool Cell::saveOdf(KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
     Validity validity = Cell(sheet(), column, row).validity();
     if (!validity.isEmpty()) {
         GenValidationStyle styleVal(&validity);
-        xmlwriter.addAttribute("table:validation-name", tableContext.valStyle.lookup(styleVal));
+        xmlwriter.addAttribute("table:validation-name", tableContext.valStyle.insert(styleVal));
     }
     if (isFormula()) {
         //kDebug(36003) <<"Formula found";
@@ -1155,9 +1179,18 @@ bool Cell::saveOdf(KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
     }
 
     if (!isEmpty() && link().isEmpty()) {
-        xmlwriter.startElement("text:p");
-        xmlwriter.addTextNode(displayText().toUtf8());
-        xmlwriter.endElement();
+        QSharedPointer<QTextDocument> doc = richText();
+        if (doc) {
+            KoEmbeddedDocumentSaver saver;
+            KoShapeSavingContext shapeContext(xmlwriter, mainStyles, saver);
+            KoTextWriter writer(shapeContext);
+
+            writer.write(doc.data(), 0);
+        } else {
+            xmlwriter.startElement("text:p");
+            xmlwriter.addTextNode(displayText().toUtf8());
+            xmlwriter.endElement();
+        }
     }
 
     // flake
@@ -1264,17 +1297,15 @@ void Cell::saveOdfValue(KoXmlWriter &xmlWriter)
 
 bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
 {
-    kDebug(36003) << "*** Loading cell properties ***** at" << name();
-
     //Search and load each paragraph of text. Each paragraph is separated by a line break.
-    loadOdfCellText(element);
+    loadOdfCellText(element, tableContext);
 
     //
     // formula
     //
     bool isFormula = false;
     if (element.hasAttributeNS(KoXmlNS::table, "formula")) {
-        kDebug(36003) << " formula :" << element.attributeNS(KoXmlNS::table, "formula", QString());
+        kDebug(36003) << "cell:" << name() << "formula :" << element.attributeNS(KoXmlNS::table, "formula", QString());
         isFormula = true;
         QString oasisFormula(element.attributeNS(KoXmlNS::table, "formula", QString()));
         // each spreadsheet application likes to safe formulas with a different namespace
@@ -1298,7 +1329,7 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
     //
     if (element.hasAttributeNS(KoXmlNS::table, "validation-name")) {
         const QString validationName = element.attributeNS(KoXmlNS::table, "validation-name", QString());
-        kDebug(36003) << " validation-name:" << validationName;
+        kDebug(36003) << "cell:" << name() << "validation-name:" << validationName;
         Validity validity;
         validity.loadOdfValidation(this, validationName, tableContext);
         if (!validity.isEmpty())
@@ -1310,7 +1341,7 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
     //
     if (element.hasAttributeNS(KoXmlNS::office, "value-type")) {
         const QString valuetype = element.attributeNS(KoXmlNS::office, "value-type", QString());
-        kDebug(36003) << "  value-type:" << valuetype;
+        kDebug(36003) << "cell:" << name() << "value-type:" << valuetype;
         if (valuetype == "boolean") {
             const QString val = element.attributeNS(KoXmlNS::office, "boolean-value", QString()).toLower();
             if ((val == "true") || (val == "false"))
@@ -1320,9 +1351,16 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
         // integer and floating-point value
         else if (valuetype == "float") {
             bool ok = false;
-            const Value value(element.attributeNS(KoXmlNS::office, "value", QString()).toDouble(&ok));
-            if (ok)
+            Value value(element.attributeNS(KoXmlNS::office, "value", QString()).toDouble(&ok));
+            if (ok) {
+                value.setFormat(Value::fmt_Number);
                 setValue(value);
+#if 0
+                Style style;
+                style.setFormatType(Format::Number);
+                setStyle(style);
+#endif
+            }
             // always set the userInput to the actual value read from the cell, and not whatever happens to be set as text, as the textual representation of a value may be less accurate than the value itself
             if (!isFormula)
                 setUserInput(sheet()->map()->converter()->asString(value).asString());
@@ -1341,9 +1379,11 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
                     currency = Currency(element.attributeNS(KoXmlNS::office, "currency", QString()));
                 }
 
-                Style style;
-                style.setCurrency(currency);
-                setStyle(style);
+                if( style().isEmpty() ) {
+                    Style style;
+                    style.setCurrency(currency);
+                    setStyle(style);
+                }
             }
         } else if (valuetype == "percentage") {
             bool ok = false;
@@ -1362,29 +1402,21 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
             }
         } else if (valuetype == "date") {
             QString value = element.attributeNS(KoXmlNS::office, "date-value", QString());
-            kDebug(36003) << "Type: date, value:" << value;
 
             // "1980-10-15"
             int year = 0, month = 0, day = 0;
             bool ok = false;
 
             int p1 = value.indexOf('-');
-            if (p1 > 0)
+            if (p1 > 0) {
                 year  = value.left(p1).toInt(&ok);
-
-            kDebug(36003) << "year:" << value.left(p1);
-
-            int p2 = value.indexOf('-', ++p1);
-
-            if (ok)
-                month = value.mid(p1, p2 - p1).toInt(&ok);
-
-            kDebug(36003) << "month:" << value.mid(p1, p2 - p1);
-
-            if (ok)
-                day = value.right(value.length() - p2 - 1).toInt(&ok);
-
-            kDebug(36003) << "day:" << value.right(value.length() - p2 - 1);
+                if (ok) {
+                    int p2 = value.indexOf('-', ++p1);
+                    month = value.mid(p1, p2 - p1).toInt(&ok);
+                    if (ok)
+                        day = value.right(value.length() - p2 - 1).toInt(&ok);
+                }
+            }
 
             if (ok) {
                 setValue(Value(QDate(year, month, day), sheet()->map()->calculationSettings()));
@@ -1395,11 +1427,11 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
                 s.setFormatType(Format::ShortDate);
                 setStyle(s);
 #endif
-                kDebug(36003) << "Set QDate:" << year << " -" << month << " -" << day;
+                kDebug(36003) << "cell:" << name() << "Type: date, value:" << value << "Date:" << year << " -" << month << " -" << day;
             }
         } else if (valuetype == "time") {
             QString value = element.attributeNS(KoXmlNS::office, "time-value", QString());
-            kDebug(36003) << "Type: time:" << value;
+
             // "PT15H10M12S"
             int hours = 0, minutes = 0, seconds = 0;
             int l = value.length();
@@ -1417,14 +1449,11 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
                     seconds = num.toInt(&ok);
                 else
                     continue;
-
-                kDebug(36003) << "Num:" << num;
-
-                num = "";
+                //kDebug(36003) << "Num:" << num;
+                num.clear();
                 if (!ok)
                     break;
             }
-            kDebug(36003) << "Hours:" << hours << "," << minutes << "," << seconds;
 
             if (ok) {
                 // Value kval( timeToNum( hours, minutes, seconds ) );
@@ -1436,6 +1465,7 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
                 style.setFormatType(Format::Time);
                 setStyle(style);
 #endif
+                kDebug(36003) << "cell:" << name() << "Type: time:" << value << "Hours:" << hours << "," << minutes << "," << seconds;
             }
         } else if (valuetype == "string") {
             if (element.hasAttributeNS(KoXmlNS::office, "string-value")) {
@@ -1452,12 +1482,12 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
             setStyle(style);
 #endif
         } else {
-            kDebug(36003) << "  Unknown type. Parsing user input.";
+            kDebug(36003) << "cell:" << name() << "  Unknown type. Parsing user input.";
             // Set the value by parsing the user input.
             parseUserInput(userInput());
         }
     } else { // no value-type attribute
-        kDebug(36003) << "  No value type specified. Parsing user input.";
+        kDebug(36003) << "cell:" << name() << "  No value type specified. Parsing user input.";
         // Set the value by parsing the user input.
         parseUserInput(userInput());
     }
@@ -1526,13 +1556,28 @@ KoXmlElement namedItemNSWithSpan(const KoXmlNode& node, const char* nsURI, const
     return KoXmlElement();
 }
 
-void Cell::loadOdfCellText(const KoXmlElement& parent)
+// recursively goes through all children of parent and returns true if there is any element
+// in the draw: namespace in this subtree
+static bool findDrawElements(const KoXmlElement& parent)
+{
+    KoXmlElement element;
+    forEachElement(element , parent) {
+        if (element.namespaceURI() == KoXmlNS::draw)
+            return true;
+        if (findDrawElements(element))
+            return true;
+    }
+    return false;
+}
+
+void Cell::loadOdfCellText(const KoXmlElement& parent, OdfLoadingContext& tableContext)
 {
     //Search and load each paragraph of text. Each paragraph is separated by a line break
     KoXmlElement textParagraphElement;
     QString cellText;
 
     bool multipleTextParagraphsFound = false;
+    const bool hasRichText = KoTextLoader::containsRichText(parent);
 
     forEachElement(textParagraphElement , parent) {
         if (textParagraphElement.localName() == "p" &&
@@ -1565,6 +1610,21 @@ void Cell::loadOdfCellText(const KoXmlElement& parent)
     if (!cellText.isNull()) {
         setUserInput(cellText);
         // The value will be set later in loadOdf().
+
+        if (hasRichText && !findDrawElements(parent)) {
+            // for now we don't support richtext and embedded shapes in the same cell;
+            // this is because they would currently be loaded twice, once by the KoTextLoader
+            // and later properly by the cell itself
+            KoTextLoader loader(*tableContext.shapeContext);
+            QSharedPointer<QTextDocument> doc(new QTextDocument);
+            KoTextDocument(doc.data()).setStyleManager(sheet()->map()->textStyleManager());
+
+            QTextCursor cursor(doc.data());
+            loader.loadBody(parent, cursor);
+
+            setUserInput(doc->toPlainText());
+            setRichText(doc);
+        }
     }
 
     //Enable word wrapping if multiple lines of text have been found.
@@ -1590,7 +1650,7 @@ void Cell::loadOdfObjects(const KoXmlElement &parent, KoOdfLoadingContext& odfCo
                 KoXmlNS::table, "end-y",
                 "table:end-y"));
 
-    KoShapeLoadingContext shapeContext(odfContext, d->sheet->dataCenterMap());
+    KoShapeLoadingContext shapeContext(odfContext, d->sheet->resourceManager());
     KoXmlElement element;
     forEachElement(element, parent) {
         if (element.namespaceURI() != KoXmlNS::draw)
