@@ -27,7 +27,7 @@
 // koffice
 #include <KoShapeRegistry.h>
 #include <KoInlineNote.h>
-#include <KoShapeFactory.h>
+#include <KoShapeFactoryBase.h>
 #include <KoShapeContainer.h>
 #include <KoStyleManager.h>
 #include <KoParagraphStyle.h>
@@ -55,7 +55,7 @@ KWDLoader::KWDLoader(KWDocument *parent, KoStore *store)
         m_pageStyle(m_pageManager->defaultPageStyle()),
         m_foundMainFS(false)
 {
-    connect(this, SIGNAL(sigProgress(int)), m_document, SIGNAL(sigProgress(int)));
+    connect(this, SIGNAL(progressUpdate(int)), m_document, SIGNAL(sigProgress(int)));
 }
 
 KWDLoader::~KWDLoader()
@@ -66,7 +66,7 @@ bool KWDLoader::load(KoXmlElement &root)
 {
     QTime dt;
     dt.start();
-    emit sigProgress(0);
+    emit progressUpdate(0);
     kDebug(32001) << "KWDocument::loadXML";
 
     QString mime = root.attribute("mime");
@@ -81,7 +81,7 @@ bool KWDLoader::load(KoXmlElement &root)
     }
     //KWLoadingInfo *loadingInfo = new KWLoadingInfo();
 
-    emit sigProgress(5);
+    emit progressUpdate(5);
 
     KoPageLayout pgLayout = KoPageLayout::standardLayout();
     // <PAPER>
@@ -167,7 +167,7 @@ bool KWDLoader::load(KoXmlElement &root)
                 pos = KWord::FootNoteSeparatorCenter;
             else if (tmp == "right")
                 pos = KWord::FootNoteSeparatorRight;
-            else // default: if ( tmp =="left" )
+            else // default: if (tmp =="left")
                 pos = KWord::FootNoteSeparatorLeft;
             m_pageStyle.setFootNoteSeparatorLinePosition(pos);
         }
@@ -212,7 +212,7 @@ bool KWDLoader::load(KoXmlElement &root)
             m_foundMainFS = true; // we will not reuse the main FS now.
         }
 
-        //KWDocument::getAttribute( attributes, "standardpage", QString::null );
+        //KWDocument::getAttribute(attributes, "standardpage", QString::null);
         if (attributes.attribute("hasHeader") != "1") {
             m_pageStyle.setHeaderPolicy(KWord::HFTypeNone);
             if (m_firstPageStyle.isValid())
@@ -230,9 +230,9 @@ bool KWDLoader::load(KoXmlElement &root)
         //    m_document->m_tabStop = attributes.attribute("tabStopValue").toDouble();
         /* TODO
                 m_initialEditing = new InitialEditing();
-                m_initialEditing->m_initialFrameSet = attributes.attribute( "activeFrameset" );
-                m_initialEditing->m_initialCursorParag = attributes.attribute( "cursorParagraph" ).toInt();
-                m_initialEditing->m_initialCursorIndex = attributes.attribute( "cursorIndex" ).toInt();
+                m_initialEditing->m_initialFrameSet = attributes.attribute("activeFrameset");
+                m_initialEditing->m_initialCursorParag = attributes.attribute("cursorParagraph").toInt();
+                m_initialEditing->m_initialCursorIndex = attributes.attribute("cursorIndex").toInt();
         */
     }
     if (m_firstPageStyle.isValid()
@@ -253,7 +253,7 @@ bool KWDLoader::load(KoXmlElement &root)
     if (!isReadWrite())
         variableCollection()->variableSetting()->setDisplayFieldCode(false);
 
-    emit sigProgress(10);
+    emit progressUpdate(10);
 
     KoXmlElement mailmerge = root.namedItem("MAILMERGE").toElement();
     if (mailmerge != KoXmlElement()) {
@@ -261,14 +261,14 @@ bool KWDLoader::load(KoXmlElement &root)
     }
 #endif
 
-    emit sigProgress(15);
+    emit progressUpdate(15);
 
     // Load all styles before the corresponding paragraphs try to use them!
     KoXmlElement stylesElem = root.namedItem("STYLES").toElement();
     if (!stylesElem.isNull())
         loadStyleTemplates(stylesElem);
 
-    emit sigProgress(17);
+    emit progressUpdate(17);
 #if 0
 
     KoXmlElement frameStylesElem = root.namedItem("FRAMESTYLES").toElement();
@@ -277,7 +277,7 @@ bool KWDLoader::load(KoXmlElement &root)
     else // load default styles
         loadDefaultFrameStyleTemplates();
 
-    emit sigProgress(18);
+    emit progressUpdate(18);
 
     KoXmlElement tableStylesElem = root.namedItem("TABLESTYLES").toElement();
     if (!tableStylesElem.isNull())
@@ -285,11 +285,11 @@ bool KWDLoader::load(KoXmlElement &root)
     else // load default styles
         loadDefaultTableStyleTemplates();
 
-    emit sigProgress(19);
+    emit progressUpdate(19);
 
     loadDefaultTableTemplates();
 
-    emit sigProgress(20);
+    emit progressUpdate(20);
 
     KoXmlElement bookmark = root.namedItem("BOOKMARKS").toElement();
     if (!bookmark.isNull()) {
@@ -340,14 +340,14 @@ bool KWDLoader::load(KoXmlElement &root)
     }
 
 
-    emit sigProgress(25);
+    emit progressUpdate(25);
 
 
     KoXmlElement framesets = root.namedItem("FRAMESETS").toElement();
     if (!framesets.isNull())
         loadFrameSets(framesets);
 
-    emit sigProgress(85);
+    emit progressUpdate(85);
 
     insertAnchors();
     insertNotes();
@@ -356,7 +356,7 @@ bool KWDLoader::load(KoXmlElement &root)
 
     loadPictureMap(root);
 
-    emit sigProgress(90);
+    emit progressUpdate(90);
 
     // <EMBEDDED>
     loadEmbeddedObjects(root);
@@ -366,7 +366,7 @@ bool KWDLoader::load(KoXmlElement &root)
         m_pageManager->appendPage(m_pageStyle);
     }
 
-    emit sigProgress(100); // the rest is only processing, not loading
+    emit progressUpdate(100); // the rest is only processing, not loading
 
     kDebug(32001) << "Loading took" << (float)(dt.elapsed()) / 1000 << " seconds";
 
@@ -394,7 +394,7 @@ void KWDLoader::loadFrameSets(const KoXmlElement &framesets)
     }
 }
 
-void KWDLoader::loadFrameSet(const KoXmlElement &framesetElem, bool loadFrames, bool loadFootnote)
+void KWDLoader::loadFrameSet(const KoXmlElement &framesetElem)
 {
     QString fsname = framesetElem.attribute("name");
 
@@ -487,12 +487,12 @@ void KWDLoader::loadFrameSet(const KoXmlElement &framesetElem, bool loadFrames,
         fill(fs, framesetElem);
         fs->setName(fsname);
 
-        KoShapeFactory *factory = KoShapeRegistry::instance()->value("PictureShape");
+        KoShapeFactoryBase *factory = KoShapeRegistry::instance()->value("PictureShape");
         Q_ASSERT(factory);
-        KoShape *shape = factory->createDefaultShapeAndInit(m_document->dataCenterMap());
+        KoShape *shape = factory->createDefaultShape(m_document->resourceManager());
         shape->setKeepAspectRatio(image.attribute("keepAspectRatio", "true") == "true");
 
-        KoImageCollection *collection = dynamic_cast<KoImageCollection*>(m_document->dataCenterMap()["ImageCollection"]);
+        KoImageCollection *collection = m_document->resourceManager()->imageCollection();
         Q_ASSERT(collection);
         KoImageData *data = collection->createImageData(imageKey.filename, m_store);
         shape->setUserData(data);
@@ -531,8 +531,18 @@ void KWDLoader::loadFrameSet(const KoXmlElement &framesetElem, bool loadFrames,
 
 void KWDLoader::fill(KWFrameSet *fs, const KoXmlElement &framesetElem)
 {
-    //m_visible = static_cast<bool>( KWDocument::getAttribute( framesetElem, "visible", true ) ); // TODO
-    //m_protectSize=static_cast<bool>( KWDocument::getAttribute( framesetElem, "protectSize", false ) ); TODO
+    if (framesetElem.hasAttribute("visible")) {
+        bool visible = framesetElem.attribute("visible").toLower() == "true";
+        foreach (KWFrame *frame, fs->frames()) {
+            frame->shape()->setVisible(visible);
+        }
+    }
+    if (framesetElem.hasAttribute("protectSize")) {
+        bool protectSize = framesetElem.attribute("protectSize").toLower() == "true";
+        foreach (KWFrame *frame, fs->frames()) {
+            frame->shape()->setGeometryProtected(protectSize);
+        }
+    }
 
 }
 
@@ -543,9 +553,9 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
     KoXmlElement frameElem;
     forEachElement(frameElem, framesetElem) {
         if (frameElem.tagName() == "FRAME") {
-            KoShapeFactory *factory = KoShapeRegistry::instance()->value(TextShape_SHAPEID);
+            KoShapeFactoryBase *factory = KoShapeRegistry::instance()->value(TextShape_SHAPEID);
             Q_ASSERT(factory);
-            KoShape *shape = factory->createDefaultShapeAndInit(m_document->dataCenterMap());
+            KoShape *shape = factory->createDefaultShape(m_document->resourceManager());
             KWTextFrame *frame = new KWTextFrame(shape, fs);
             fill(frame, frameElem);
 
@@ -554,7 +564,7 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
     }
 
 
-    //m_info = static_cast<KWFrameSet::Info>( KWDocument::getAttribute( framesetElem, "frameInfo", KWFrameSet::FI_BODY ) ); // TODO
+    //m_info = static_cast<KWFrameSet::Info>(KWDocument::getAttribute(framesetElem, "frameInfo", KWFrameSet::FI_BODY)); // TODO
     switch (framesetElem.attribute("frameInfo").toInt()) {
     case 0: ;
     }
@@ -576,7 +586,7 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
                 cursor.insertBlock(emptyTbf, emptyCf);
             }
 
-            KoStyleManager * styleManager = dynamic_cast<KoStyleManager *>(m_document->dataCenterMap()["StyleManager"]);
+            KoStyleManager *styleManager = m_document->resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
             Q_ASSERT(styleManager);
 
             firstParag = false;
@@ -1114,7 +1124,8 @@ void KWDLoader::fill(ImageKey *key, const KoXmlElement &keyElement)
 
 void KWDLoader::loadStyleTemplates(const KoXmlElement &stylesElem)
 {
-    KoStyleManager *styleManager = dynamic_cast<KoStyleManager *>(m_document->dataCenterMap()["StyleManager"]);
+    KoStyleManager *styleManager = m_document->resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
+
     Q_ASSERT(styleManager);
 
     KoXmlElement style;
@@ -1167,7 +1178,7 @@ void KWDLoader::insertAnchors()
         if (fs->frames().count() == 0)  continue;
         KWFrame *frame = fs->frames().first();
         frame->shape()->setPosition(QPointF(0, 0));
-        KoShapeContainer* container = dynamic_cast<KoShapeContainer*>(anchor.textShape);
+        KoShapeContainer *container = dynamic_cast<KoShapeContainer*>(anchor.textShape);
         Q_ASSERT(container);
         if (! container) continue;
         container->addChild(frame->shape());   // attach here & avoid extra layouts
