@@ -208,8 +208,11 @@ QRectF KoTextDocumentLayout::frameBoundingRect(QTextFrame *frame) const
 
 int KoTextDocumentLayout::hitTest(const QPointF &point, Qt::HitTestAccuracy accuracy) const
 {
-    return hitTestIterated(document()->rootFrame()->begin(), 
+    int position = hitTestIterated(document()->rootFrame()->begin(),
                         document()->rootFrame()->end(), point, accuracy);
+    if (accuracy != Qt::ExactHit && position == -1)
+        return document()->rootFrame()->lastPosition();
+    return position;
 }
 
 int KoTextDocumentLayout::hitTestIterated(QTextFrame::iterator begin, QTextFrame::iterator end, const QPointF &point, Qt::HitTestAccuracy accuracy) const
@@ -219,14 +222,23 @@ int KoTextDocumentLayout::hitTestIterated(QTextFrame::iterator begin, QTextFrame
     for (it = begin; it != end; ++it) {
         QTextBlock block = it.currentBlock();
         QTextTable *table = qobject_cast<QTextTable*>(it.currentFrame());
+        QTextFrame *subFrame = it.currentFrame();
 
         if (table) {
             QTextTableCell cell = m_state->hitTestTable(table, point);
             if (cell.isValid()) {
-                return hitTestIterated(cell.begin(), cell.end(), point,
+                position = hitTestIterated(cell.begin(), cell.end(), point,
                                 accuracy);
+                if (position == -1)
+                    position = cell.lastPosition();
+                return position;
             }
             continue;
+        } else if (subFrame) {
+            position = hitTestIterated(subFrame->begin(), subFrame->end(), point, accuracy);
+            if (position != -1)
+                return position;
+            continue;
         } else {
             if (!block.isValid())
                 continue;
@@ -234,7 +246,7 @@ int KoTextDocumentLayout::hitTestIterated(QTextFrame::iterator begin, QTextFrame
         // kDebug(32500) <<"hitTest[" << point.x() <<"," << point.y() <<"]";
         QTextLayout *layout = block.layout();
         if (point.y() > layout->boundingRect().bottom()) {
-            position = block.position() + block.length() - 1;
+            // just skip this block. position = block.position() + block.length() - 1;
             continue;
         }
         for (int i = 0; i < layout->lineCount(); i++) {
@@ -255,11 +267,8 @@ int KoTextDocumentLayout::hitTestIterated(QTextFrame::iterator begin, QTextFrame
             }
             return block.position() + line.xToCursor(point.x());
         }
-        
     }
-    if (accuracy == Qt::ExactHit)
-        return -1;
-    return position;
+    return -1;
 }
 
 int KoTextDocumentLayout::pageCount() const
@@ -399,8 +408,10 @@ void KoTextDocumentLayout::layout()
                 m_state->shape->update(QRectF(0, posY,
                                               m_state->shape->size().width(), m_state->y() - posY));
 
-            if (! moreText)
+            if (! moreText) {
+                emit finishedLayout();
                 return; // done!
+            }
             continue;
         }
         if (m_state->numColumns() > 0)
@@ -411,6 +422,7 @@ void KoTextDocumentLayout::layout()
         while (m_state->addLine(line)) {
             if (m_state->shape == 0) { // shape is full!
                 line.setPosition(QPointF(0, m_state->y() + 20));
+                emit finishedLayout();
                 return; // done!
             }
             line.setLineWidth(m_state->width());
